@@ -43,7 +43,18 @@ class AuthProvider(object):
         if self.check_credentials(credentials):
             self.credentials = credentials
         else:
-            raise TypeError("Invalid credentials")
+            if isinstance(credentials, Credentials):
+                password = credentials.get('password')
+                message = "Credentials are: " + str(credentials)
+                if password is None:
+                    message += " Password is not defined."
+                else:
+                    message += " Password is defined."
+                raise exceptions.InvalidCredentials(message)
+            else:
+                raise TypeError("credentials object is of type %s, which is"
+                                " not a valid Credentials object type." %
+                                credentials.__class__.__name__)
         self.cache = None
         self.alt_auth_data = None
         self.alt_part = None
@@ -248,7 +259,7 @@ class KeystoneAuthProvider(AuthProvider):
 class KeystoneV2AuthProvider(KeystoneAuthProvider):
 
     def _auth_client(self, auth_url):
-        return json_v2id.TokenClientJSON(
+        return json_v2id.TokenClient(
             auth_url, disable_ssl_certificate_validation=self.dsvm,
             ca_certs=self.ca_certs, trace_requests=self.trace_requests)
 
@@ -326,7 +337,7 @@ class KeystoneV2AuthProvider(KeystoneAuthProvider):
 class KeystoneV3AuthProvider(KeystoneAuthProvider):
 
     def _auth_client(self, auth_url):
-        return json_v3id.V3TokenClientJSON(
+        return json_v3id.V3TokenClient(
             auth_url, disable_ssl_certificate_validation=self.dsvm,
             ca_certs=self.ca_certs, trace_requests=self.trace_requests)
 
@@ -517,11 +528,13 @@ class Credentials(object):
             if key in self.ATTRIBUTES:
                 setattr(self, key, attr[key])
             else:
-                raise exceptions.InvalidCredentials
+                msg = '%s is not a valid attr for %s' % (key, self.__class__)
+                raise exceptions.InvalidCredentials(msg)
 
     def __str__(self):
         """Represent only attributes included in self.ATTRIBUTES"""
-        _repr = dict((k, getattr(self, k)) for k in self.ATTRIBUTES)
+        attrs = [attr for attr in self.ATTRIBUTES if attr is not 'password']
+        _repr = dict((k, getattr(self, k)) for k in attrs)
         return str(_repr)
 
     def __eq__(self, other):
