@@ -18,6 +18,8 @@ import os
 import shlex
 import subprocess
 
+import six
+
 from tempest_lib import base
 import tempest_lib.cli.output_parser
 from tempest_lib import exceptions
@@ -50,7 +52,9 @@ def execute(cmd, action, flags='', params='', fail_ok=False,
     cmd = ' '.join([os.path.join(cli_dir, cmd),
                     flags, action, params])
     LOG.info("running: '%s'" % cmd)
-    cmd = shlex.split(cmd.encode('utf-8'))
+    if six.PY2:
+        cmd = cmd.encode('utf-8')
+    cmd = shlex.split(cmd)
     result = ''
     result_err = ''
     stdout = subprocess.PIPE
@@ -62,7 +66,10 @@ def execute(cmd, action, flags='', params='', fail_ok=False,
                                        cmd,
                                        result,
                                        result_err)
-    return result
+    if six.PY2:
+        return result
+    else:
+        return os.fsdecode(result)
 
 
 class CLIClient(object):
